@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2021 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -93,8 +93,8 @@ static bool findBackward (const QTextDocument *txtdoc, const QString &str,
 /*************************/
 // This method extends the searchable strings to those with line breaks.
 // It also corrects the behavior of Qt's backward search.
-QTextCursor FN::finding (const QString& str,
-                         const QTextCursor& start,
+QTextCursor FN::finding (const QString &str,
+                         const QTextCursor &start,
                          QTextDocument::FindFlags flags) const
 {
     /* let's be consistent first */
@@ -376,7 +376,7 @@ void FN::find()
     QModelIndex nxtIndx;
     if (found.isNull())
     {
-        if (!ui->everywhereButton->isChecked() || model_->rowCount() == 1)
+        if (!ui->everywhereButton->isChecked())
         {
             if (backwardSearch)
                 start.movePosition (QTextCursor::End, QTextCursor::MoveAnchor);
@@ -400,7 +400,11 @@ void FN::find()
                     if (!backwardSearch)
                         nxtIndx = model_->index (0, 0);
                     else
+                    {
                         nxtIndx = model_->index (model_->rowCount() - 1, 0);
+                        while (model_->hasChildren (nxtIndx))
+                            nxtIndx = model_->index (model_->rowCount (nxtIndx) - 1, 0, nxtIndx);
+                    }
                 }
                 DomItem *item = static_cast<DomItem*>(nxtIndx.internalPointer());
                 if (TextEdit *thisTextEdit = widgets_.value (item))
