@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2020 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2021 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,7 +27,6 @@ void handleQuitSignals (const std::vector<int>& quitSignals)
 {
     auto handler = [](int sig) ->void {
         Q_UNUSED (sig);
-        //printf("\nUser signal = %d.\n", sig);
         QCoreApplication::quit();
     };
 
@@ -38,7 +37,7 @@ void handleQuitSignals (const std::vector<int>& quitSignals)
 int main(int argc, char *argv[])
 {
     const QString name = "FeatherNotes";
-    const QString version = "0.8.0";
+    const QString version = "0.10.0";
     const QString option = QString::fromUtf8 (argv[1]);
     if (option == "--help" || option == "-h")
     {
@@ -47,9 +46,9 @@ int main(int argc, char *argv[])
                "\nUsage:\n	feathernotes [options] [file] "\
                "Open the specified file\nOptions:\n"\
                "--version or -v   Show version information and exit.\n"\
-               "--help            Show this help and exit\n"\
-               "-m, --min         Start minimized\n"\
-               "-t, --tray        Start iconified to tray if there is a tray icon\n\n";
+               "--help            Show this help and exit.\n"\
+               "-m, --min         Start minimized.\n"\
+               "-t, --tray        Start iconified to tray if there is a tray.\n\n";
         return 0;
     }
     else if (option == "--version" || option == "-v")
@@ -72,7 +71,7 @@ int main(int argc, char *argv[])
     handleQuitSignals ({SIGQUIT, SIGINT, SIGTERM, SIGHUP}); // -> https://en.wikipedia.org/wiki/Unix_signal
 #endif
 
-    app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
+    app.setAttribute (Qt::AA_UseHighDpiPixmaps, true);
 
     QStringList langs (QLocale::system().uiLanguages());
     QString lang; // bcp47Name() doesn't work under vbox
@@ -92,7 +91,7 @@ int main(int argc, char *argv[])
 
     QTranslator FPTranslator;
 #if defined (Q_OS_HAIKU)
-    FPTranslator.load ("feathernotes_" + lang, "/translations");
+    FPTranslator.load ("feathernotes_" + lang, qApp->applicationDirPath() + "/translations");
 #elif defined (Q_OS_WIN)
     FPTranslator.load ("feathernotes_" + lang, qApp->applicationDirPath() + "\\..\\data\\translations\\translations");
 #else
@@ -109,7 +108,8 @@ int main(int argc, char *argv[])
     }
 
     FeatherNotes::FN w (message);
-    //w.show();
+
+    QObject::connect (&app, &QCoreApplication::aboutToQuit, &w, &FeatherNotes::FN::quitting);
 
     return app.exec();
 }
