@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2019 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2019-2021 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -40,14 +40,16 @@ FNKeySequenceEdit::FNKeySequenceEdit (QWidget *parent) : QKeySequenceEdit (paren
 void FNKeySequenceEdit::keyPressEvent (QKeyEvent *event)
 { // also a workaround for a Qt bug that makes Meta a non-modifier
     clear(); // no multiple shortcuts
+    int k = event->key();
+    if ((event->modifiers() != Qt::NoModifier && event->modifiers() != Qt::KeypadModifier))
+    {
+        if (k == Qt::Key_Super_L || k == Qt::Key_Super_R)
+            return;
+    }
     /* don't create a shortcut without modifier because
        this is a text editor but make exceptions for Fx keys */
-    int k = event->key();
-    if ((k < Qt::Key_F1 || k > Qt::Key_F35)
-        && (event->modifiers() == Qt::NoModifier || event->modifiers() == Qt::KeypadModifier))
-    {
+    else if (k < Qt::Key_F1 || k > Qt::Key_F35)
         return;
-    }
     QKeySequenceEdit::keyPressEvent (event);
 }
 /*************************/
@@ -63,16 +65,23 @@ QWidget* Delegate::createEditor (QWidget *parent,
 /*************************/
 bool Delegate::eventFilter (QObject *object, QEvent *event)
 {
-    QWidget *editor = qobject_cast<QWidget*>(object);
+    FNKeySequenceEdit *editor = qobject_cast<FNKeySequenceEdit*>(object);
     if (editor && event->type() == QEvent::KeyPress)
     {
-        int k = static_cast<QKeyEvent *>(event)->key();
+        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+        int k = ke->key();
         if (k == Qt::Key_Return || k == Qt::Key_Enter)
         {
             emit QAbstractItemDelegate::commitData (editor);
             emit QAbstractItemDelegate::closeEditor (editor);
             return true;
         }
+        /* treat Tab and Backtab like other keys */
+        if (k == Qt::Key_Tab || k ==  Qt::Key_Backtab)
+        {
+            editor->pressKey (ke);
+            return true;
+        }
     }
     return QStyledItemDelegate::eventFilter (object, event);
 }
@@ -132,13 +141,7 @@ PrefDialog::PrefDialog (QWidget *parent)
         /* splitter position */
         ui->splitterSizeBox->setChecked (win->isSplitterRem());
         connect (ui->splitterSizeBox, &QCheckBox::stateChanged, win, [win] (int checked) {
-            if (checked == Qt::Checked)
-            {
-                win->remSplitter (true);
-                win->setSplitterSizes (win->getSpltiterState());
-            }
-            else if (checked == Qt::Unchecked)
-                win->remSplitter (false);
+            win->remSplitter (checked == Qt::Checked);
         });
 
         /* window position */
@@ -317,6 +320,17 @@ PrefDialog::PrefDialog (QWidget *parent)
             win->setOpenLastFile (checked == Qt::Checked);
         });
 
+        /* the number of recently opend files */
+        ui->recentSpin->setValue (win->getRecentFilesNumber());
+        connect (ui->recentSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), win, [win] (int value) {
+            win->setRecentFilesNumber (value); // will take effect at closeEvent()
+        });
+        /* whether recent files should be opened separately */
+        ui->recentBox->setChecked (win->getOpenReccentSeparately()); // up-to-date because of FN::getRecentFilesNumber()
+        connect (ui->recentBox, &QCheckBox::stateChanged, win, [win] (int checked) {
+            win->setOpenReccentSeparately (checked == Qt::Checked);
+        });
+
         /* spell checking */
 #ifdef HAS_HUNSPELL
         ui->dictEdit->setText (win->getDictPath());
@@ -470,8 +484,8 @@ void PrefDialog::onClosing()
         win->setPrefSize (size());
 
         win->writeConfig();
-        win->writeGeometryConfig();
-        win->rememberLastOpenedFile();
+        win->writeGeometryConfig (false); // doesn't save the last node info
+        win->rememberLastOpenedFile (true); // saves everything about recent files
     }
 }
 /*************************/
