@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2020 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2021 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -43,15 +43,10 @@ TextEdit::TextEdit (QWidget *parent) : QTextEdit (parent)
     textTab_ = "    "; // the default text tab is four spaces
     pressPoint = QPoint();
     scrollTimer_ = nullptr;
+    isCopyOrCut_ = false;
 
     VScrollBar *vScrollBar = new VScrollBar;
     setVerticalScrollBar (vScrollBar);
-
-#if (QT_VERSION == QT_VERSION_CHECK(5,14,0))
-    /* workaround */
-    HScrollBar *hScrollBar = new HScrollBar;
-    setHorizontalScrollBar (hScrollBar);
-#endif
 }
 /*************************/
 TextEdit::~TextEdit()
@@ -70,11 +65,7 @@ QString TextEdit::remainingSpaces (const QString& spaceTab, const QTextCursor& c
     QTextCursor tmp = cursor;
     QString txt = cursor.block().text().left (cursor.positionInBlock());
     QFontMetricsF fm = QFontMetricsF (document()->defaultFont());
-#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
     qreal spaceL = fm.horizontalAdvance (" ");
-#else
-    qreal spaceL = fm.width (" ");
-#endif
     int n = 0, i = 0;
     while ((i = txt.indexOf("\t", i)) != -1)
     { // find tab widths in terms of spaces
@@ -111,11 +102,7 @@ QTextCursor TextEdit::backTabCursor (const QTextCursor& cursor, bool twoSpace) c
 
     QString txt = blockText.left (indx);
     QFontMetricsF fm = QFontMetricsF (document()->defaultFont());
-#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
     qreal spaceL = fm.horizontalAdvance (" ");
-#else
-    qreal spaceL = fm.width (" ");
-#endif
     int n = 0, i = 0;
     while ((i = txt.indexOf("\t", i)) != -1)
     { // find tab widths in terms of spaces
@@ -163,13 +150,38 @@ static inline bool isOnlySpaces (const QString &str)
 
 void TextEdit::keyPressEvent (QKeyEvent *event)
 {
-    if (event->modifiers() == Qt::ControlModifier && !isReadOnly() && event->key() == Qt::Key_Z)
-    { // only for a workaround (see TextEdit::undo)
+    /* overriding copy() and cut() */
+    if (event == QKeySequence::Copy)
+    {
+        copy();
+        event->accept();
+        return;
+    }
+    if (event == QKeySequence::Cut)
+    {
+        cut();
+        event->accept();
+        return;
+    }
+
+    /* QWidgetTextControl::undo() and QWidgetTextControl::redo() call ensureCursorVisible()
+       even when there's nothing to undo/redo. Users may press the undo/redo shortcut keys
+       just to make sure of the state of a document and a scroll jump can confuse them when
+       there's nothing to undo/redo. */
+    if (event == QKeySequence::Undo)
+    {
         if (document()->isUndoAvailable())
             undo();
         event->accept();
         return;
     }
+    if (event == QKeySequence::Redo)
+    {
+        if (document()->isRedoAvailable())
+            redo();
+        event->accept();
+        return;
+    }
 
     if (event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter)
     {
@@ -479,6 +491,30 @@ void TextEdit::keyPressEvent (QKeyEvent *event)
             return;
         }
     }
+    else if (event->key() == Qt::Key_Down || event->key() == Qt::Key_Up)
+    {
+        if (event->modifiers() == Qt::ControlModifier)
+        {
+            if (QScrollBar* vbar = verticalScrollBar())
+            { // scroll without changing the cursor position
+                vbar->setValue(vbar->value() + (event->key() == Qt::Key_Down ? 1 : -1) * vbar->singleStep());
+                event->accept();
+                return;
+            }
+        }
+    }
+    else if (event->key() == Qt::Key_PageDown || event->key() == Qt::Key_PageUp)
+    {
+        if (event->modifiers() == Qt::ControlModifier)
+        {
+            if (QScrollBar* vbar = verticalScrollBar())
+            { // scroll without changing the cursor position
+                vbar->setValue(vbar->value() + (event->key() == Qt::Key_PageDown ? 1 : -1) * vbar->pageStep());
+                event->accept();
+                return;
+            }
+        }
+    }
     else if (event->key() == Qt::Key_Tab)
     {
         QTextCursor cursor = textCursor();
@@ -564,7 +600,7 @@ void TextEdit::keyPressEvent (QKeyEvent *event)
         }
 
     }
-    /* because of a bug in Qt5, the non-breaking space (ZWNJ) isn't inserted with SHIFT+SPACE */
+    /* because of a bug in Qt, the non-breaking space (ZWNJ) may not be inserted with SHIFT+SPACE */
     else if (event->key() == 0x200c)
     {
         insertPlainText (QChar (0x200C));
@@ -720,6 +756,24 @@ bool TextEdit::event (QEvent *e)
     return QTextEdit::event (e);
 }
 /*************************/
+QMimeData* TextEdit::createMimeDataFromSelection() const
+{
+    /* this is for giving only plain text to the selection clipboard
+       and leaving the main clipboard to QTextEdit */
+    if (!isCopyOrCut_)
+    {
+        QTextCursor cursor = textCursor();
+        if (cursor.hasSelection())
+        {
+            QMimeData *mimeData = new QMimeData;
+            mimeData->setText (cursor.selection().toPlainText());
+            return mimeData;
+        }
+        return nullptr;
+    }
+    return QTextEdit::createMimeDataFromSelection();
+}
+/*************************/
 bool TextEdit::canInsertFromMimeData (const QMimeData *source) const
 {
     if (source->hasImage() || source->hasUrls())
@@ -793,48 +847,55 @@ void TextEdit::mousePressEvent (QMouseEvent *e)
             && e->buttons() == Qt::LeftButton)
         {
             tripleClickTimer_.invalidate();
-            QTextCursor txtCur = textCursor();
-            const QString blockText = txtCur.block().text();
-            const int l = blockText.length();
-            txtCur.movePosition (QTextCursor::StartOfBlock);
-            int i = 0;
-            while (i < l && blockText.at (i).isSpace())
-                ++i;
-            /* WARNING: QTextCursor::movePosition() can be a mess with RTL
-                        but QTextCursor::setPosition() works fine. */
-            if (i < l)
+            if (!(qApp->keyboardModifiers() & Qt::ControlModifier))
             {
-                txtCur.setPosition (txtCur.position() + i);
-                int j = l;
-                while (j > i && blockText.at (j -  1).isSpace())
-                    --j;
-                txtCur.setPosition (txtCur.position() + j - i, QTextCursor::KeepAnchor);
+                QTextCursor txtCur = textCursor();
+                const QString blockText = txtCur.block().text();
+                const int l = blockText.length();
+                txtCur.movePosition (QTextCursor::StartOfBlock);
+                int i = 0;
+                while (i < l && blockText.at (i).isSpace())
+                    ++i;
+                /* WARNING: QTextCursor::movePosition() can be a mess with RTL
+                            but QTextCursor::setPosition() works fine. */
+                if (i < l)
+                {
+                    txtCur.setPosition (txtCur.position() + i);
+                    int j = l;
+                    while (j > i && blockText.at (j -  1).isSpace())
+                        --j;
+                    txtCur.setPosition (txtCur.position() + j - i, QTextCursor::KeepAnchor);
+                    setTextCursor (txtCur);
+                }
+                if (txtCur.hasSelection())
+                {
+                    QClipboard *cl = QApplication::clipboard();
+                    if (cl->supportsSelection())
+                    {
+                        if (QMimeData *data = createMimeDataFromSelection())
+                            cl->setMimeData (data, QClipboard::Selection);
+                    }
+                }
+                e->accept();
+                return;
             }
-            else
-                txtCur.setPosition (txtCur.position() + i, QTextCursor::KeepAnchor);
-            setTextCursor (txtCur);
-            return;
         }
-        tripleClickTimer_.invalidate();
+        else
+            tripleClickTimer_.invalidate();
     }
 
     QTextEdit::mousePressEvent (e);
-    pressPoint = e->pos();
+
+    if (e->button() == Qt::LeftButton)
+        pressPoint = e->pos();
 }
 
 /*************************/
 void TextEdit::mouseReleaseEvent (QMouseEvent *e)
 {
     QTextEdit::mouseReleaseEvent (e);
-
-    /* give only a plain text to the selection clipboard */
-    QTextCursor cursor = textCursor();
-    if (cursor.hasSelection())
-    {
-        QClipboard *cl = QApplication::clipboard();
-        if (cl->supportsSelection())
-            cl->setText (cursor.selection().toPlainText(), QClipboard::Selection);
-    }
+    if (e->button() != Qt::LeftButton)
+        return;
 
     QString str = anchorAt (e->pos());
     if (!str.isEmpty()
@@ -859,9 +920,10 @@ void TextEdit::mouseDoubleClickEvent (QMouseEvent *e)
 /*************************/
 void TextEdit::wheelEvent (QWheelEvent *e)
 {
+    QPoint deltaPoint = e->angleDelta();
     if (e->modifiers() & Qt::ControlModifier)
     {
-        float delta = e->angleDelta().y() / 120.f;
+        float delta = deltaPoint.y() / 120.f;
         zooming (delta);
         return;
     }
@@ -869,25 +931,34 @@ void TextEdit::wheelEvent (QWheelEvent *e)
     /* smooth scrolling */
     if (e->spontaneous() && e->source() == Qt::MouseEventNotSynthesized)
     {
-#if (QT_VERSION >= QT_VERSION_CHECK(5,12,0))
-        bool horizontal (e->angleDelta().x() != 0);
-#else
-        bool horizontal (e->orientation() == Qt::Horizontal);
-#endif
-        QScrollBar* sbar = horizontal
-                               ? horizontalScrollBar() : verticalScrollBar();
-        if (sbar)
+        bool horizontal (qAbs (deltaPoint.x()) > qAbs (deltaPoint.y()));
+        QScrollBar* sbar = horizontal ? horizontalScrollBar()
+                                      : verticalScrollBar();
+        if (sbar && sbar->isVisible())
         {
-            int delta = horizontal
-                            ? e->angleDelta().x() : e->angleDelta().y();
-            if (e->modifiers() & Qt::ShiftModifier) // scrolling with minimum speed
-                delta /= QApplication::wheelScrollLines();
+            int delta = horizontal ? deltaPoint.x() : deltaPoint.y();
             if ((delta > 0 && sbar->value() == sbar->minimum())
                 || (delta < 0 && sbar->value() == sbar->maximum()))
             {
                 return; // the scrollbar can't move
             }
 
+            if (QApplication::wheelScrollLines() > 1)
+            {
+                if ((e->modifiers() & Qt::ShiftModifier)
+                    || qAbs (delta) < 120) // touchpad
+                { // scrolling with minimum speed
+                    if (qAbs (delta) >= scrollAnimFrames * QApplication::wheelScrollLines())
+                        delta /= QApplication::wheelScrollLines();
+                }
+            }
+
+            /* wait until the angle delta reaches an acceptable value */
+            static int _delta = 0;
+            _delta += delta;
+            if (abs(_delta) < scrollAnimFrames)
+                return;
+
             if (!scrollTimer_)
             {
                 scrollTimer_ = new QTimer();
@@ -897,11 +968,13 @@ void TextEdit::wheelEvent (QWheelEvent *e)
 
             /* set the data for inertial scrolling */
             scrollData data;
-            data.delta = delta;
+            data.delta = _delta;
             data.leftFrames = scrollAnimFrames;
             data.vertical = !horizontal;
             queuedScrollSteps_.append (data);
-            scrollTimer_->start (1000 / SCROLL_FRAMES_PER_SEC);
+            if (!scrollTimer_->isActive())
+                scrollTimer_->start (1000 / SCROLL_FRAMES_PER_SEC);
+            _delta = 0;
             return;
         }
     }
@@ -919,6 +992,8 @@ void TextEdit::scrollSmoothly()
     {
         int delta = qRound (static_cast<qreal>(it->delta) / static_cast<qreal>(scrollAnimFrames));
         int remainingDelta = it->delta - (scrollAnimFrames - it->leftFrames) * delta;
+        if ((delta >= 0 && remainingDelta < 0) || (delta < 0 && remainingDelta >= 0))
+            remainingDelta = 0;
         if (qAbs (delta) >= qAbs (remainingDelta))
         { // this is the last frame or, due to rounding, there can be no more frame
             if (it->vertical)
@@ -938,61 +1013,37 @@ void TextEdit::scrollSmoothly()
         }
     }
 
-    if (totalDeltaH != 0 && horizontalScrollBar())
+    if (totalDeltaH != 0)
     {
-#if (QT_VERSION >= QT_VERSION_CHECK(5,12,0))
-        QWheelEvent eventH (QPointF(),
-                            QPointF(),
-                            QPoint(),
-#if (QT_VERSION == QT_VERSION_CHECK(5,14,0))
-                            QPoint (totalDeltaH, 0),
-#else
-                            QPoint (0, totalDeltaH),
-#endif
-                            Qt::NoButton,
-                            Qt::NoModifier,
-                            Qt::NoScrollPhase,
-                            false);
-#else
-        QWheelEvent eventH (QPointF(),
-                            QPointF(),
-                            totalDeltaH,
-                            Qt::NoButton,
-                            Qt::NoModifier,
-                            Qt::Vertical);
-#endif
-        QCoreApplication::sendEvent (horizontalScrollBar(), &eventH);
+        QScrollBar *hbar = horizontalScrollBar();
+        if (hbar && hbar->isVisible())
+        {
+            QWheelEvent eventH (QPointF(),
+                                QPointF(),
+                                QPoint(),
+                                QPoint (0, totalDeltaH),
+                                Qt::NoButton,
+                                Qt::NoModifier,
+                                Qt::NoScrollPhase,
+                                false);
+            QCoreApplication::sendEvent (hbar, &eventH);
+        }
     }
-    if (totalDeltaV != 0 && verticalScrollBar())
+    if (totalDeltaV != 0)
     {
-#if (QT_VERSION >= QT_VERSION_CHECK(5,12,0))
-        QWheelEvent eventV (QPointF(),
-                            QPointF(),
-                            QPoint(),
-                            QPoint (0, totalDeltaV),
-                            Qt::NoButton,
-                            Qt::NoModifier,
-                            Qt::NoScrollPhase,
-                            false);
-#else
-        QWheelEvent eventV (QPointF(),
-                            QPointF(),
-                            totalDeltaV,
-                            Qt::NoButton,
-                            Qt::NoModifier,
-                            Qt::Vertical);
-#endif
-        QCoreApplication::sendEvent (verticalScrollBar(), &eventV);
-    }
-
-    /* update text selection if the left mouse button is pressed (-> QPlainTextEdit::timerEvent) */
-    if (QApplication::mouseButtons() & Qt::LeftButton)
-    {
-        const QPoint globalPos = QCursor::pos();
-        QPoint pos = viewport()->mapFromGlobal (globalPos);
-        QMouseEvent ev (QEvent::MouseMove, pos, viewport()->mapTo (viewport()->topLevelWidget(), pos), globalPos,
-                        Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
-        mouseMoveEvent (&ev);
+        QScrollBar *vbar = verticalScrollBar();
+        if (vbar && vbar->isVisible())
+        {
+            QWheelEvent eventV (QPointF(),
+                                QPointF(),
+                                QPoint(),
+                                QPoint (0, totalDeltaV),
+                                Qt::NoButton,
+                                Qt::NoModifier,
+                                Qt::NoScrollPhase,
+                                false);
+            QCoreApplication::sendEvent (vbar, &eventV);
+        }
     }
 
     if (queuedScrollSteps_.empty())
@@ -1007,38 +1058,26 @@ void TextEdit::zooming (float range)
     if (newSize <= 0) return;
     f.setPointSizeF (static_cast<qreal>(newSize));
     setFont (f);
-#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
     QFontMetricsF metrics (f);
     setTabStopDistance (4 * metrics.horizontalAdvance (' '));
-#elif (QT_VERSION >= QT_VERSION_CHECK(5,10,0))
-    QFontMetricsF metrics (f);
-    setTabStopDistance (4 * metrics.width (' '));
-#else
-    QFontMetrics metrics (f);
-    setTabStopWidth (4 * metrics.width (' '));
-#endif
 
     /* if this is a zoom-out, the text will need
        to be formatted and/or highlighted again */
     if (range < 0) emit zoomedOut (this);
 }
 /*************************/
-void TextEdit::undo()
+// See createMimeDataFromSelection() for the reason.
+void TextEdit::copy()
 {
-    QTextEdit::undo();
-    /* This is a workaround for a weird Qt bug: If some text is pasted at the document start,
-       the pasting is undone and then, some text is pasted at the document start again, the
-       text color of the document stylesheet will be ignored. */
-    if (CSSTextColor.isValid())
-    {
-        QTextCursor cur = textCursor();
-        if (cur.atStart() && cur.atEnd())
-        {
-            QTextCharFormat fmt;
-            fmt.setForeground (CSSTextColor);
-            cur.setBlockCharFormat (fmt);
-        }
-    }
+    isCopyOrCut_ = true;
+    QTextEdit::copy();
+    isCopyOrCut_ = false;
+}
+void TextEdit::cut()
+{
+    isCopyOrCut_ = true;
+    QTextEdit::cut();
+    isCopyOrCut_ = false;
 }
 
 }
